local lib = TeamEquipCompare;
local libSerialize = LibStub:GetLibrary("AceSerializer-3.0");

-- methods that should be part of lua
function keys(t, sorted)
	local result = {};

	for k,v in pairs(t) do result[#result + 1] = k end
	if (sorted) then sort(result); end

	return result;
end

function vals(t)
	local result = {};

	for k,v in pairs(t) do result[#result + 1] = v end

	return result;
end

function containskey(t, key)
	if (type(t) == 'table') then
		return (t[key] ~= nil);
	else
		return false;
	end
end
-- end lua-esque methods

function TEC_RequestPartyData()
	if (IsInGroup()) then
		lib:notificationMsg("Requesting data from party...");
		lib:SendCommMessage(lib.commPrefix, "REQUEST", "PARTY");
	else
		lib:notificationMsg("Unable to request team member data -- not in a party");
	end
end

function TEC_RequestUnitData(name)
	lib:notificationMsg(format("Requesting data from '%s'...", name));
	lib:SendCommMessage(lib.commPrefix, "REQUEST", "WHISPER", name);
end

function TEC_RemoveUnitData(name)
	lib:SendCommMessage(lib.commPrefix, "REMOVE="..name, "PARTY");
end

function TEC_SendPlayerData(sendTo)
	local data = TEC_GetUnitData(lib.player.name);

	if (sendTo == "PARTY") then
		lib:notificationMsg("Sending data to party...");
		lib:SendCommMessage(lib.commPrefix, TEC_PackageData(data), "PARTY");
	else
		lib:notificationMsg(format("Sending data to '%s'...", sendTo));
		lib:SendCommMessage(lib.commPrefix, TEC_PackageData(data), "WHISPER", sendTo);
	end
end

function TEC_GetUnitData(unit)
	lib:debugMsg("Building unit data...");

	local ilvlTotal, ilvlEquipped = GetAverageItemLevel();
	local t = {
		name = lib.player.name,
		class = lib.player.class,
		level = UnitLevel("player"),
		ilevel = {
			total = ilvlTotal,
			equipped = ilvlEquipped
		},
		gear = {},
	};

	for ii = 1, 18 do
		link = GetInventoryItemLink(unit, ii);
		if (link ~= nil) then
			t.gear[ii] = GetInventoryItemLink(unit, ii);
		end
	end

	lib:debugMsg(format("Player inspection completed (%d items)", #t.gear));

	return t;
end

function TEC_PrintTeam()
	if (not IsInGroup()) then
		lib:Print("No team member data found -- not in group");
	else
		local names =  keys(lib.party, true);
		if (#names == 0) then
			lib:Print("No team member data found -- type '/tec sync' to force a data sync");
		else
			for index, name in pairs(names) do
				local data = lib.party[name];
				if (data) then
					lib:Print(format("|%s%s|r: ilevel=%d, %d items", TEC_ClassColor_Lookup[lib.party[name].class], name, lib.party[name].ilevel.equipped, #lib.party[name].gear));
				else
					lib:Print(format("|cff70b8ff%s|r: no data", name));
				end
			end
		end
	end
end

function TEC_CheckTeamMembers()
	local count = GetNumGroupMembers();
	local unitsToAdd = {};
	local unitsToRemove = {};
	
	-- first check if local player is included
	if (not containskey(lib.party, lib.player.name)) then
		unitsToAdd[#unitsToAdd + 1] = lib.player.name;
	end

	-- next check for each party member
	for ii=1,count-1 do
		local name = GetUnitName("party"..tostring(ii));
		if (not containskey(lib.party, name)) then
			unitsToAdd[#unitsToAdd + 1] = name;
		end
	end

	-- find orphans
	for unit, _ in pairs(lib.party) do
		if (not UnitInParty(unit)) then
			unitsToRemove[#unitsToRemove + 1] = unit;
		end
	end
	return unitsToAdd, unitsToRemove;
end

function TEC_PackageData(data)
	local serialized = libSerialize:Serialize(data);
	return serialized;
end

function TEC_UnpackageData(packaged)
	local success, data = libSerialize:Deserialize(packaged);
	if (success) then
		return data;
	else
		return "";
	end
end
