local lib = TeamEquipCompare;
local lineAdded = false;

local function TEC_OnTooltipSetItem(tooltip, ...)
	if (not lineAdded) and ((lib.db.global.requireAltKey) and IsAltKeyDown()) then
		-- get info for tooltip item
		tt_name, tt_link = tooltip:GetItem();
		_, _, _, _, tt_reqLevel, tt_class, tt_subclass, _, tt_equipSlot, _, _ = GetItemInfo(tt_link);
		local slot = TEC_Slot_Lookup[tt_equipSlot];

		-- simple message if player is not in a group
		if (not IsInGroup()) then
			tooltip:AddLine(" ");
			tooltip:AddDoubleLine("Team Equip Compare:", "|cffff0000[Not in Group]");
			tooltip:AddLine(" ");

		-- make sure the slot is one we're interested in
		elseif (slot and slot.ids) then
			-- add heading line with spacing
			tooltip:AddLine(" ");
			tooltip:AddLine("Team Equip Compare:");
			tooltip:AddLine(" ");

			-- loop thru party members in alphabetical order
			local sortedNames = keys(lib.party, true);
			for _, unit in pairs(sortedNames) do
				local class = TEC_Class_Lookup[lib.party[unit].class];
				local classColor = TEC_ClassColor_Lookup[lib.party[unit].class];
				local isUsable = TEC_ItemIsUsable(lib.party[unit], class, tt_reqLevel, tt_class, tt_subclass);
				local playerTag = format("|%s%s:", classColor, lib.party[unit].name);

				if (not isUsable) then
					-- add player name with note that tooltip item is unusable
					tooltip:AddDoubleLine(playerTag, "|cffff0000[Unable to Equip]");
					tooltip:AddLine(" ");
				else
					-- add party member heading
					tooltip:AddLine(playerTag);

					-- compare item with each potential slot (weapons, trinkets and fingers have two slots)
					for ii = 1, #slot.ids do
						-- grab item from syncronized party member data
						local link = lib.party[unit].gear[slot.ids[ii]];
						if (link) then
							-- get party member item data
							local name, _, quality, ilevel, _, _, _, _, _, _, _ = GetItemInfo(link);
							if (name) then
								-- get item text, colored by item quality
								local itemTag = format("     |%s[%d] %s", TEC_QualityColor_Lookup[quality], ilevel, name)
								local statTip = "";

								-- get stat comparison values, if item is usable by the party member
								if (isUsable) then
									local statDelta = GetItemStatDelta(tt_link, link);
									for stat, value in pairs(statDelta) do
										-- separate stat items with a comma
										if (#statTip > 0) then statTip = statTip .. ", " end
										
										-- get friendly name string from lookup table
										statName = TEC_Stat_Lookup[stat];
										if (not statName) then statName = tostring(stat) end

										-- color comparison text depending on +/-
										if (value > 0) then
											statTip = statTip..format("|cff00ff00+%d|cffffffff %s", value, statName);
										elseif (value < 0) then
											statTip = statTip..format("|cffff0000%d|cffffffff %s", value, statName);
										end
									end
								elseif (link == TEC_link) then
									statTip = "[Equipped]";
								end
								
								-- add item line to the tooltip
								tooltip:AddDoubleLine(itemTag, statTip);
							end
						end
					end

					-- add spacer between party members
					tooltip:AddLine(" ");
				end
			end
		end
	end

	lineAdded = true;
end

local function TEC_OnTooltipCleared(tooltip, ...)
	lineAdded = false;
end

function TEC_ItemIsUsable(unit, class, reqLevel, itemClass, itemSubclass)
	if (reqLevel > unit.level) then
		return false;
	elseif (not class) then
		return false;
	elseif (not class[itemClass]) then
		return false;
	elseif (not class[itemClass][itemSubclass]) then
		return false;
	elseif (class[itemClass][itemSubclass] > unit.level) then
		return false;
	else
		return true;
	end
end

-- hook into the game tooltip
GameTooltip:HookScript("OnTooltipSetItem", TEC_OnTooltipSetItem);
GameTooltip:HookScript("OnTooltipCleared", TEC_OnTooltipCleared);
