--[[
-- TODO: add money
-- TODO: update data when a toon levels
-- TODO: add comments
-- TODO: make chat messages consistent
-- TODO: refactor tooltip methods

-- TODO: only request data from a new party member, on the group update event

-- TODO: what to do re: enable/disable?

-- TODO: look into tooltip layout with LibQTip library or another library to make it look better
-- TODO: add localization

-- TODO: tabbed dialog
	-- party member comparison
		-- stats? class? spec?
	-- gear window for each toon -- can I make it like the normal gear window?
]]

TeamEquipCompare = LibStub("AceAddon-3.0"):NewAddon("TeamEquipCompare", "AceConsole-3.0", "AceComm-3.0", "AceEvent-3.0");
local lib = TeamEquipCompare;
local defaultDB = {
	global = {
		showDebugMsgs = false,
		showNotificationMsgs = true,
		requireAltKey = true
	}
};

lib.version = "0.3";
lib.commPrefix = "TEC_COMMS";
lib.player = {};
lib.party = {};

function lib:notificationMsg(msg)
	if (lib.db.global.showNotificationMsgs) then self:Print(msg) end
end

function lib:debugMsg(msg)
	if (lib.db.global.showDebugMsgs) then self:Print("[DEBUG]: "..msg) end
end

function lib:OnInitialize()
	-- load default global saved variable
	lib.db = LibStub("AceDB-3.0"):New("TEC_DB", defaultDB, true);

	-- load player data
	lib.player.name = GetUnitName("player", false);
	lib.player.class = UnitClass("player");

	-- register communication channel
	self:RegisterComm(lib.commPrefix, "OnCommReceived");

	-- register events
	self:RegisterEvent("GROUP_ROSTER_UPDATE", "OnGroupRosterUpdate");
	self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED", "OnPlayerEquipmentChanged");
	
	self:Print(format("TeamEquipCompare v%s loaded", lib.version));
end

-- TODO: do I need to handle enabling/disabling addon?
-- TODO: is there a default enabled variable to use?
function lib:OnEnable()
	self:Print(format("TeamEquipCompare v%s enabled", lib.version));
end

function lib:OnDisable()
	self:Print(format("TeamEquipCompare v%s disabled", lib.version));
end

function lib:OnGroupRosterUpdate(...)
	self:debugMsg("OnGroupRosterUpdate Fired");

	-- clear data if not grouped
	if (not IsInGroup()) then
		self:debugMsg("Clearing party data...");
		lib.party = {};

	-- otherwise, update the data as needed
	else
		local unitsToAdd, unitsToRemove = TEC_CheckTeamMembers();

		-- add any new party members
		if (#unitsToAdd > 0) then
			for ii=1, #unitsToAdd do
				-- add key to party table to prevent multiple requests
				lib.party[unitsToAdd[ii]] = { player = { name = unitsToAdd[ii] } };
				TEC_RequestUnitData(unitsToAdd[ii]);
			end
		end

		-- remove orphaned party members
		if (#unitsToRemove > 0) then
			for ii=1, #unitsToRemove do
				TEC_RemoveUnitData(unitsToRemove[ii]);
			end
		end
	end
end

function lib:OnPlayerEquipmentChanged(event, slotid, ...)
	self:debugMsg(format("PlayerEquipmentChanged Event Fired, slotid: %d", slotid));

	-- report my data to team if my equipment changed
	if (IsInGroup()) then
		TEC_SendPlayerData("PARTY");
	end
end

function lib:OnCommReceived(_, commMsg, _, unit, ...)
	self:debugMsg(format("%s: %s", unit, tostring(commMsg)));

	-- request to send player data to the party
	if (commMsg == "REQUEST") then
		TEC_SendPlayerData(unit);

	-- remove player data
	elseif (strfind(commMsg, "REMOVE")) then
		local unitToRemove = strmatch(commMsg, "REMOVE=(.*)");
		self:notificationMsg(format("Removing '%s' from team...", unitToRemove));
		lib.party[unitToRemove] = nil;

	-- otherwise, data sent from a team member
	else
		local data = TEC_UnpackageData(commMsg);
		if (data ~= "") then
			lib.party[unit] = data;
			self:notificationMsg(format("Received update from '%s'", unit));
		else
			self:notificationMsg(format("Error receiving data from %s", unit));
		end
	end
end
